# urls.py

from django.urls import path
from django.views.decorators.csrf import csrf_exempt

from .views import microservice_admin_view


urlpatterns = [
    path('microservice_admin/', csrf_exempt(microservice_admin_view), name='microservice_admin_view'),
]


# Generated by Django 3.2.7 on 2022-10-11 10:08

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AdminApps',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app_name', models.CharField(max_length=50, unique=True)),
                ('app_url', models.CharField(max_length=250)),
                ('redirect_url', models.CharField(max_length=250)),
                ('app_order', models.IntegerField()),
            ],
            options={
                'verbose_name': 'Admin App',
                'verbose_name_plural': 'Admin Apps',
            },
        ),
    ]



from django.contrib import admin

from .jwt_serializer import jwt_serializer
from . import models
from django.contrib import admin
from django.conf import settings
from rest_framework_simplejwt.backends import TokenBackend
from django.urls import reverse



class ModelsAdmin(admin.AdminSite):

    def each_context(self, request):
        context = super().each_context(request)

        microservice_admin_list = []
        for app in models.AdminApps.objects.all().order_by('app_order'):
            app_url = f'{app.app_url}{app.redirect_path}'
            if app.app_name == settings.MICROSERVICE_ADMIN_APP_NAME:  
                app_url = f'{app.app_url}{reverse("admin:index")}'
                microservice_admin_list.append({'app_name': app.app_name, 'app_url': app_url, 'active': True})
            else:
                microservice_admin_list.append({'app_name': app.app_name, 'app_url': app_url, 'active': False})

        context["admin_apps"]=  microservice_admin_list
        context['microservice_header'] = settings.MICROSERVICE_ADMIN_APP_SECRET_KEY_HEADER
        jwt = jwt_serializer.get_token(request.user)
        context['microservice_secret_key'] = jwt
    
        return context

admin_site = ModelsAdmin(name=settings.MICROSERVICE_ADMIN_APP_NAME)



from rest_framework_simplejwt.serializers import TokenObtainPairSerializer


class jwt_serializer(TokenObtainPairSerializer):
    @classmethod
    def get_token(cls, user):
        token = super().get_token(user)
        return token


from email.policy import default
from django.db import models
from django.conf import settings
from .admin import admin_site



class AdminApps(models.Model):
    app_name = models.CharField(max_length=50, unique=True)
    app_url = models.CharField(max_length=250, unique=True, null=True, blank=True)
    redirect_path = models.CharField(max_length=250, null=True, blank=True)
    app_order = models.IntegerField(default=0)

    def __str__(self):
        return self.app_name
    
    class Meta:
        verbose_name = 'Admin App'
        verbose_name_plural = 'Admin Apps'

admin_site.register(AdminApps)


# Generated by Django 3.2.7 on 2022-10-11 10:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_microservice_admin', '0002_auto_20221011_1009'),
    ]

    operations = [
        migrations.RenameField(
            model_name='adminapps',
            old_name='redirect_url',
            new_name='redirect_path',
        ),
    ]



# Generated by Django 3.2.7 on 2022-10-11 10:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_microservice_admin', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='adminapps',
            name='app_order',
            field=models.IntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='adminapps',
            name='app_url',
            field=models.CharField(blank=True, max_length=250, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='adminapps',
            name='redirect_url',
            field=models.CharField(blank=True, max_length=250, null=True, unique=True),
        ),
    ]



from setuptools import setup, find_packages
from glob import glob


setup(
    name='django-microservice-admin',
    version='0.1.1',
    license='MIT',
    author="Mina Atef",
    author_email='mina.atef0@gmail.com',
    packages=['django_microservice_admin', 'django_microservice_admin.migrations', 'django_microservice_admin.templates'],
    data_files = [
        ('migrations', glob('migrations/*')), 
        ('templates', glob('templates/*')), 
    ],
    
    url='https://github.com/minaaaatef/Django-Microservice-Admin',
    keywords=('django','buttons','admin','actions'),
    install_requires=[
      ],


)


import secrets
from django.conf import settings
from django.conf.urls import url
from django.shortcuts import render, redirect, HttpResponse
from django.contrib.auth import login
from django.contrib.auth import get_user_model
from django.urls import reverse
from .models import AdminApps
from rest_framework_simplejwt.backends import TokenBackend

def microservice_admin_view(request):
    if request.method == 'POST':
        if settings.MICROSERVICE_ADMIN_APP_SECRET_KEY_HEADER not in  request.POST:
            return HttpResponse('No Key', status=403)

        jwt_secret_key = request.POST[settings.MICROSERVICE_ADMIN_APP_SECRET_KEY_HEADER]
        
        try:
            valid_data = TokenBackend(
                    algorithm='HS256',signing_key=settings.SECRET_KEY).decode(jwt_secret_key, verify=True)

        except:
            return HttpResponse('Invalid Key', status=403)

        user = get_user_model().objects.get(id=valid_data['user_id'])

        login(request, user)

        return redirect(reverse('admin:index'))


from django.db import OperationalError
from django.db import transaction
from django.apps import AppConfig
from django.conf import settings
import sys
from django.urls import reverse
from django.urls.exceptions import NoReverseMatch

class MicroserviceAdminConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_microservice_admin'

    @transaction.atomic
    def ready(self):

        if 'makemigrations' in sys.argv or 'migrate' in sys.argv:
            return

        from .models import AdminApps

        MICROSERVICE_ADMIN_APP_VARS = ['MICROSERVICE_ADMIN_APP_NAME', 'MICROSERVICE_ADMIN_APP_HOST', 'MICROSERVICE_ADMIN_APP_ORDER']

        for var in MICROSERVICE_ADMIN_APP_VARS:
            if not hasattr(settings, var):
                raise Exception(f"Missing {var} in settings.py")
        

            AdminApps.objects.get_or_create(
                app_name=settings.MICROSERVICE_ADMIN_APP_NAME,
            )
            try:
                AdminApps.objects.filter(app_name=settings.MICROSERVICE_ADMIN_APP_NAME).update(
                    app_url=settings.MICROSERVICE_ADMIN_APP_HOST.rstrip('/'),
                    redirect_path = reverse('microservice_admin_view'),
                    app_order=settings.MICROSERVICE_ADMIN_APP_ORDER,
                )
            except NoReverseMatch:
                raise Exception(f"Missing reverse('microservice_admin_view') in urls.py, please add it to your urls.py")